#!/usr/bin/perl

# Copyright (C) 2003 by Johannes Grassler. All rights reserved. This is
# free software and can be distributed under the terms of either the
# Artistic License or the GNU General Public License which can be found
# in the files Artistic and COPYING.

use strict;
use warnings;

use Getopt::Std;
use Digest::MD5 qw(md5 md5_hex);

# ***************************************
# *** begin of user configurable part ***

# Set the GID for the sum affected users

my $target_gid = 2001;

# Create the list of users and groups that are to be skipped when
# iterating through the passwd database
my @protected_groups = qw(admins);
my @protected_users;

# **** end of user configurable part ****
# ***************************************

my %opts;
getopts('a:i:o:h', \%opts);

my $input = "list_current";
if ( $opts{i} ) { $input = $opts{i}; }

my $output = "list_create";
if ( $opts{o} ) { $output = $opts{o}; }

my $usage = "usage: $0 -i <input file> -o <output file> -a <file with checksums to ignore>\n";
if ( $opts{h} ) { die $usage; }


# Read the list of MD5 sums to be checked against
my %users;
open INPUT,$input or die "Couldn't open $input: $!\n";
while ( my $line = <INPUT> )
	{
	chomp $line;
	my ($user, $realname, $class, $checksum) = split(/\t/, $line);
	$users{$checksum}= 
			{ 
			'name' => $user,
			'realname' => $realname,
			'class' => $class,
			'checksum' => $checksum
			};
	}

my @ignoresums;
if ( $opts{a} ) 
	{ 
	my $ignoresums_file;
	$ignoresums_file = $opts{a};

	# get the checksums that are to be ignored
	open IGNORESUMS,$ignoresums_file or die "Couldn't open $ignoresums_file: $!\n" ;
	while ( my $line = <IGNORESUMS> )
		{
		chomp $line;
		push @ignoresums, $line;
		}
	close IGNORESUMS;
	}


# open the output filehandle

open OUTPUT,">$output";

# Iterate through the passwd file
PWDPARSE: while ( my ($username, undef, undef, $gid, undef, undef, $gecos) = getpwent() )
	{
	# First check if the user is in the target group and skip to the
	# next iteration if not
	if ( $gid ne $target_gid ) { next PWDPARSE; }

	# Skip protected users
	foreach my $skipuser (@protected_users)
		{
		if ( $username eq $skipuser ) { next PWDPARSE; }
		}

	# Now check the sum of the user entry against the sum from the list
	my (undef, undef, $sum) = split(/;/, $gecos);
	if ( $users{$sum} ) { delete $users{$sum}; }

	foreach my $adminsum (@ignoresums)
		{
		# Prevent the admins' accounts from being created a second time
		if ( $users{$adminsum} ) { delete $users{$adminsum}; }
		}
	}

foreach my $newuser ( keys %users )
	{
	print OUTPUT "$users{$newuser}{'name'}\t$users{$newuser}{'realname'}\t$users{$newuser}{'class'}\t$users{$newuser}{'checksum'}\n";
	}
